@@ -3,22 +3,21 @@
 view_dataset.py
 """
 
-from rich.console import Console
-
-CONSOLE = Console(width=120)
 import time
 from datetime import timedelta
 from pathlib import Path
 
 import torch
 import tyro
+from rich.console import Console
 
 from nerfstudio.configs.base_config import ViewerConfig
-from nerfstudio.data.datamanagers import AnnotatedDataParserUnion
+from nerfstudio.data.datamanagers.base_datamanager import AnnotatedDataParserUnion
 from nerfstudio.data.datasets.base_dataset import InputDataset
 from nerfstudio.viewer.server import viewer_utils
 
 DEFAULT_TIMEOUT = timedelta(minutes=30)
+CONSOLE = Console(width=120)
 
 # speedup for when input size to model doesn't change (much)
 torch.backends.cudnn.benchmark = True  # type: ignore
@@ -30,9 +29,10 @@ def main(
     log_base_dir: Path = Path("/tmp/nerfstudio_viewer_logs"),
 ) -> None:
     """Main function."""
-    viewer_state = viewer_utils.ViewerState(
+    viewer_state, _ = viewer_utils.setup_viewer(
         viewer,
         log_filename=log_base_dir / viewer.relative_log_filename,
+        datapath=dataparser.data,
     )
     dataset = InputDataset(dataparser.setup().get_dataparser_outputs(split="train"))
     viewer_state.init_scene(dataset=dataset, start_train=False)
